var Dragon = Backbone.Model.extend({
    initialize: function(gameteA, gameteB) {
        if(typeof gameteA == 'undefined') {
            var chromosomes = [
                this.randomChromosome(),
                this.randomChromosome()
            ];
            // Set the gender specifically
            chromosomes[0][this.genepool.SEX] = this.genepool.pool[this.genepool.SEX][0];
            if(Math.floor(Math.random() * 2) + 1 == 1) {
                chromosomes[1][this.genepool.SEX] = this.genepool.pool[this.genepool.SEX][0];
            } else {
                chromosomes[1][this.genepool.SEX] = this.genepool.pool[this.genepool.SEX][1];
            }
        } else {
            var chromosomes = [ gameteA, gameteB ]
        }
        this.set('chromosomes', chromosomes);
    },

    genepool: {
        FIRE: 0, HORNS: 1, CLAWS: 2, BODY: 3, WINGS: 4, COLOR: 5, SEX: 6,
        pool: [
            [
                { name: 'F', description: 'Firebreathing', isDominant: true },
                { name: 'f', description: 'Fire resistance', isDominant: false }
            ],
            [
                { name: 'H', description: 'Horns', isDominant: true },
                { name: 'h', description: 'No horns', isDominant: false }
            ],
            [
                { name: 'C', description: 'Claws', isDominant: true },
                { name: 'c', description: 'No claws', isDominant: false }
            ],
            [
                { name: 'B', description: 'Large body', isDominant: true },
                { name: 'b', description: 'Slender body', isDominant: false }
            ],
            [
                { name: 'W', description: 'Large wings', isDominant: true },
                { name: 'w', description: 'Small wings', isDominant: false }
            ],
            [
                { 
                    name: 'R', description: 'Red color', isDominant: true, 
                    codominantDescription: 'Brown color', codominantName: 'B'
                },
                { 
                    name: 'G', description: 'Green color', isDominant: true, 
                    codominantDescription: 'Brown color', codominantName: 'B'
                },
                { name: 'g', description: 'Gold color', isDominant: false }
            ],
            [
                { name: 'X', description: 'Female', isDominant: false },
                { name: 'Y', description: 'Male', isDominant: true }
            ]
        ],
    },

    randomChromosome : function() {
        var chromosome = Array();
        for(var i = 0; i < this.genepool.pool.length; i++) {
            chromosome.push(
                this.genepool.pool[i][
                    Math.floor(Math.random() * 
                    this.genepool.pool[i].length)
                ]
            );
        }
        return chromosome;
    },

    getPhenotype: function(num) {
        if(this.get('chromosomes')[0][num].name == this.get('chromosomes')[1][num].name) { // Same genotype
            return this.get('chromosomes')[0][num].name;
        } else {
            if(
                this.get('chromosomes')[0][num].isDominant && 
                this.get('chromosomes')[1][num].isDominant
            ) { // Codominance
                return this.get('chromosomes')[0][num].codominantName;
            } else {
                if(this.get('chromosomes')[0][num].isDominant) { // Chromosome 0 is dominant
                    return this.get('chromosomes')[0][num].name;
                } else if(this.get('chromosomes')[1][num].isDominant) { // Chromosome 1 is dominant
                    return this.get('chromosomes')[1][num].name;
                } else { // Both are resessive
                    return this.get('chromosomes')[0][num].name;
                }
            }
        }
    },

    getGenotype: function(num) {
        return this.get('chromosomes')[0][num].name + this.get('chromosomes')[1][num].name;
    },
    
    produceGamete: function() {
        var gamete = Array();
        for(var i = 0; i < this.get('chromosomes')[0].length; i++) {
            gamete.push(
                this.get('chromosomes')[Math.floor(Math.random() * this.get('chromosomes').length)][i]
            );
        }
        return gamete;
    }
});

var DragonView = Backbone.View.extend({
    className: 'dragon',
    initialize: function() {
        if(this.model.getPhenotype(this.model.genepool.COLOR) == 'B') {
            var color = 'brown';
        } else if(this.model.getPhenotype(this.model.genepool.COLOR) == 'G') {
            var color = 'green';
        } else if(this.model.getPhenotype(this.model.genepool.COLOR) == 'R') {
            var color = 'red';
        } else if(this.model.getPhenotype(this.model.genepool.COLOR) == 'g') {
            var color = 'gold';
        }
        if(this.model.getPhenotype(this.model.genepool.BODY) == 'B') {
            background = 'url(images/dragon-' + color + '.png) no-repeat 0px 0px, ';
        } else {
            background = 'url(images/dragon-' + color + '.png) no-repeat 0px -200px, ';
        }
        if(this.model.getPhenotype(this.model.genepool.HORNS) == 'H') {
            background += 'url(images/dragon-horns.png), ';
        }
        if(this.model.getPhenotype(this.model.genepool.CLAWS) == 'C') {
            background += 'url(images/dragon-claws.png), ';
        }
        if(this.model.getPhenotype(this.model.genepool.FIRE) == 'F') {
            background += 'url(images/dragon-fire.png) no-repeat 0px 0px, ';
        } else {
            background += 'url(images/dragon-fire.png) no-repeat 0px -200px, ';
        }
        if(this.model.getPhenotype(this.model.genepool.WINGS) == 'W') {
            background += 'url(images/dragon-wings.png) no-repeat 0px 0px, ';
        } else {
            background += 'url(images/dragon-wings.png) no-repeat 0px -200px, ';
        }
        if(this.model.getPhenotype(this.model.genepool.SEX) == 'X') {
            background += 'url(images/dragon-sex.png) no-repeat 0px 0px';
        } else {
            background += 'url(images/dragon-sex.png) no-repeat 0px -200px';
        }
        var dragonImage = $('<div class="dragon-image" />');
        dragonImage.css('background', background);
        this.$el.append(dragonImage);
    },
    events: {
        'click button#select-for-breeding' : 'selectForBreeding',
        'click button#delete' : 'deleteDragon',
        'click button#show-genes' : 'showGenes',
    },
    addActionButtons: function() {
        this.addGenes();
        this.$el.append('<button id="select-for-breeding">Select for Breeding</button>');
        this.$el.append('<button id="delete">Delete</button>');
    },
    addGenes: function() {
        this.genes = new Field({ className: 'genes' });
        var chromosome1 = new Field({ className: 'chromosome' });
        var chromosome2 = new Field({ className: 'chromosome' });
        for(var i = 0; i < this.model.genepool.pool.length; i++) {
            var allele1 = $('<div class="allele allele' + i + '" />')
                .text(this.model.get('chromosomes')[0][i].name);
            chromosome1.$el.append(allele1);
            var allele1 = $('<div class="allele allele' + i + '" />')
                .text(this.model.get('chromosomes')[1][i].name);
            chromosome2.$el.append(allele1);
        }
        this.genes.$el.append(chromosome1.el);
        this.genes.$el.append(chromosome2.el);
        this.$el.append(this.genes.el);

        this.$el.append('<button id="show-genes">Show Genes</button');
    },
    deleteDragon: function() {
        this.$el.remove();
    },
    selectForBreeding: function() {
        eventsDispatcher.trigger('selectForBreeding', this.model);
    },
    showGenes: function() {
        this.genes.$el.toggle();
    }
});
