var GameModel = Backbone.Model.extend({
});
var GameView = Backbone.View.extend({
    el: $('.container'),
    model: new GameModel(),
    initialize: function() {
        // Set up all the fields
        this.infoField = new Field({ className: 'info-field' });
        this.goalField = new Field({ className: 'goal-field' });
        this.fatherField = new Field({ className: 'father-field' });
        this.motherField = new Field({ className: 'mother-field' });
        this.matingField = new Field({ className : 'mating-field' });
        this.breedingField = new Field({ className : 'breeding-field' });
        eventsDispatcher.on('selectForBreeding', this.addToMatingField, this);
        this.render();
    },
    events: {
        'click button#mate' : 'mateDragons',
        'click button#get-new-dragon' : 'getNewDragon'
    },
    render: function() {
        // Set up the goal
        var goal = new DragonView({ model: new Dragon() });
        this.model.set('goal', goal.model);
        this.goalField.$el.append('<h2>Goal</h2>');
        this.goalField.$el.append(goal.el);
        this.infoField.$el.append(this.goalField.el);

        // Set up the mating fields
        this.matingField.$el.append(this.fatherField.el);
        this.matingField.$el.append(this.motherField.el);
        this.infoField.$el.append('<button id="get-new-dragon">Get New Dragon</button>');
        this.infoField.$el.append(this.matingField.el);

        // Set up the breeding fields
        for(var i = 0; i < 4; i++) {
            var dragon = new DragonView({ model: new Dragon() });
            dragon.addActionButtons();
            this.breedingField.$el.append(dragon.el);
        }
        this.breedingField.$el.append(dragon.el);
        this.$el.append(this.infoField.el);
        this.$el.append(this.breedingField.el);
        return this;
    },
    addToMatingField: function(model) {
        var dragon = new DragonView({ model: model });
        if(model.getPhenotype(model.genepool.SEX) == 'Y') { // Male
            this.fatherField.$el.html(dragon.el);
            this.model.set('father', model);
        } else {
            this.motherField.$el.html(dragon.el);
            this.model.set('mother', model);
        }
        if(this.model.get('father') && this.model.get('mother') && $('button#mate').length == 0) {
            this.matingField.$el.append('<button id="mate">Mate</button>');
        }
    },
    mateDragons: function() {
        if(
            typeof(this.model.get('father')) != 'undefined' &&
            typeof(this.model.get('mother')) != 'undefined'
        ) {
            var fatherDragon = this.model.get('father');
            var motherDragon = this.model.get('mother');
            var babyDragon = new DragonView({ 
                model: new Dragon(fatherDragon.produceGamete(), motherDragon.produceGamete()) 
            });
            this.goalCheck(babyDragon.model);
            babyDragon.addActionButtons();
            this.breedingField.$el.append(babyDragon.el);
            this.fatherField.$el.empty();
            this.motherField.$el.empty();
            this.model.set('mother', null);
            this.model.set('father', null);
            this.matingField.$el.find('button#mate').remove();
        }
    },
    goalCheck: function(testDragon) {
        var win = true;
        var goalDragon = this.model.get('goal');
        for(var i = 0; i < goalDragon.genepool.pool.length; i++) {
            if(testDragon.getPhenotype(i) != goalDragon.getPhenotype(i)) {
                win = false;
            }
        }
        if(win) {
            alert('You win!');
        }
    },
    getNewDragon: function() {
        var newDragon = new DragonView({ model: new Dragon() });
        newDragon.addActionButtons();
        this.breedingField.$el.append(newDragon.el);
    }
});

var Field = Backbone.View.extend({
    tagName : 'div'
});

var eventsDispatcher = _.extend({}, Backbone.Events);

var game = new GameView();
